from __future__ import annotations

import json

from django_spire.knowledge.entry.version.block.blocks.text_block import TextBlock
from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.models import EntryVersionBlock
from django_spire.knowledge.entry.version.tests.factories import \
    create_test_entry_version


def create_test_version_block(**kwargs) -> EntryVersionBlock:
    data = {
        'version': create_test_entry_version(),
        'order': 1,
        'type': BlockTypeChoices.TEXT,
        '_block_data': json.dumps({}),
        '_text_data': ''
    }
    data.update(kwargs)
    version_block = EntryVersionBlock.objects.create(**data)
    version_block.block = TextBlock(value='', type=BlockTypeChoices.TEXT)
    version_block.save()

    return version_block



import django_glue as dg

from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib import Breadcrumbs
from django_spire.contrib.form.utils import show_form_errors
from django_spire.contrib.generic_views import portal_views
from django_spire.core.shortcuts import get_object_or_null_obj
from django_spire.file.interfaces import MultiFileUploader
from django_spire.knowledge.collection.models import Collection
from django_spire.knowledge.entry.models import Entry
from django_spire.knowledge.entry.forms import EntryForm, EntryFilesForm
from django_spire.knowledge.entry.version.maps import FILE_TYPE_CONVERTER_MAP


@AppAuthController('knowledge').permission_required('can_add')
def form_view(
        request: WSGIRequest,
        collection_pk: int,
        pk: int = 0,
) -> TemplateResponse | HttpResponseRedirect:
    entry = get_object_or_null_obj(Entry, pk=pk)
    collection = Collection.objects.get(pk=collection_pk)

    dg.glue_model_object(request, 'entry', entry, fields=['name'])

    if request.method == 'POST':
        form = EntryForm(request.POST, instance=entry)

        if form.is_valid():
            form.cleaned_data['collection'] = collection
            entry, _ = entry.services.save_model_obj(author=request.user, **form.cleaned_data)

            return HttpResponseRedirect(
                reverse(
                    'django_spire:knowledge:entry:version:page:detail',
                    kwargs={'pk': entry.pk}
                )
            )

        show_form_errors(request, form)
    else:
        form = EntryForm(instance=entry)

    return portal_views.form_view(
        request,
        form=form,
        obj=entry,
        context_data={
            'entry': entry,
            'action_url': (
                reverse(
                    'django_spire:knowledge:entry:form:create',
                    kwargs={'collection_pk': collection_pk}
                )
                if pk == 0
                else reverse(
                    'django_spire:knowledge:entry:form:update',
                    kwargs={'pk': pk, 'collection_pk': collection_pk}
                )
            )
        },
        template='django_spire/knowledge/entry/page/form_page.html'
    )


@AppAuthController('knowledge').permission_required('can_add')
def import_form_view(
        request: WSGIRequest,
        collection_pk: int
) -> TemplateResponse | HttpResponseRedirect:
    dg.glue_query_set(
        request,
        'collections',
        Collection.objects.active(),
        fields=['name']
    )

    if request.method == 'POST':
        file_form = EntryFilesForm(request.POST, request.FILES)

        if file_form.is_valid():
            file_uploader = MultiFileUploader(
                related_field=None,
                app_name='knowledge'
            )
            file_objects = file_uploader.upload(request.FILES.getlist('import_files'))

            _ = Entry.services.factory.create_from_files(
                author=request.user,
                collection=Collection.objects.get(pk=collection_pk),
                files=file_objects
            )
            return HttpResponseRedirect(
                reverse(
                    'django_spire:knowledge:entry:template:file_list',
                    kwargs={'collection_pk': collection_pk}
                )
            )

        show_form_errors(request, file_form)

    breadcrumbs = Breadcrumbs()
    breadcrumbs.add_breadcrumb(name='Import Files')
    supported_file_types = [
        '.' + file_type for file_type in list(FILE_TYPE_CONVERTER_MAP.keys())
    ]

    return portal_views.template_view(
        request,
        breadcrumbs=breadcrumbs,
        page_title='Import Files',
        page_description='Import Files',
        context_data={
            'collection_pk': collection_pk,
            'supported_file_types': supported_file_types,
            'supported_file_types_verbose': ', '.join(supported_file_types)
        },
        template='django_spire/knowledge/entry/page/import_form_page.html'
    )



from __future__ import annotations

from django.contrib import admin
from django.urls import reverse
from django.utils.html import format_html

import django_spire.auth.user.models
from django_spire.auth.group import models
from django_spire.auth.user.tools import add_user_to_all_user_group


@admin.register(models.AuthGroup)
class PortalGroupAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'view_group_detail_link', 'user_count')
    search_fields = ('id', 'name')
    ordering = ('name',)

    def view_group_detail_link(self, group: models.AuthGroup) -> str:
        url = reverse('django_spire:auth:group:page:detail', kwargs={'pk': group.pk})
        return format_html(f'<a href="{url}">View Details</a>')

    view_group_detail_link.short_description = 'Details Link'

    def user_count(self, group: models.AuthGroup) -> int:
        return group.user_set.count()

    user_count.short_description = 'User Count'


@admin.register(django_spire.auth.user.models.AuthUser)
class PortalUserAdmin(admin.ModelAdmin):
    actions = ['add_to_all_user_group']
    list_display = ('id', 'username', 'email', 'full_name', 'is_active', 'view_user_profile_link')
    list_filter = ('is_active', 'is_staff')
    search_fields = ('id', 'username', 'email', 'first_name', 'last_name')
    ordering = ('username',)

    def full_name(self, user: django_spire.auth.user.models.AuthUser) -> str:
        return user.get_full_name()

    full_name.short_description = 'Full Name'

    def view_user_profile_link(self, user: django_spire.auth.user.models.AuthUser) -> str:
        url = reverse('django_spire:auth:user:page:detail', kwargs={'pk': user.pk})
        return format_html(f'<a href="{url}">Profile</a>')

    view_user_profile_link.short_description = 'Profile Link'

    @admin.action(description='Add to All Users Group')
    def add_to_all_user_group(self, request, queryset):
        for user in queryset:
            add_user_to_all_user_group(user)
        self.message_user(request, f'Updated {len(queryset)} users.')



from __future__ import annotations

import string

from unittest.mock import patch

from django.template import Context, RequestContext, Template
from django.test import RequestFactory, TestCase

from django_spire.core.templatetags.spire_core_tags import (
    add_str,
    content_type_url,
    in_list,
    index,
    generate_id,
    not_in_list,
    query_param_url,
    to_snake_case,
)


class TemplateTagUnitTests(TestCase):
    def setUp(self) -> None:
        super().setUp()

        self.factory = RequestFactory()

    def test_add_str(self) -> None:
        self.assertEqual(add_str('Hello', 'World'), 'HelloWorld')

    def test_in_list(self) -> None:
        self.assertTrue(in_list('a', 'a,b,c'))
        self.assertFalse(in_list('d', 'a,b,c'))

    def test_index(self) -> None:
        items = [10, 20, 30]

        self.assertEqual(index(items, 1), 20)
        self.assertEqual(index(items, 5), items)

    def test_generate_id(self) -> None:
        identifier = generate_id()

        self.assertEqual(len(identifier), 8)
        self.assertTrue(all(ch in string.ascii_letters for ch in identifier))

    def test_not_in_list(self) -> None:
        self.assertTrue(not_in_list('x', 'a,b,c'))
        self.assertFalse(not_in_list('a', 'a,b,c'))

    def test_to_snake_case(self) -> None:
        self.assertEqual(to_snake_case('Hello World'), 'hello_world')

    def test_content_type_url(self) -> None:
        class Dummy:
            pass

        dummy = Dummy()

        dummy._meta = type(
            'meta',
            (),
            {'app_label': 'myapp', 'model_name': 'dummy'}
        )

        func = 'django_spire.core.templatetags.spire_core_tags.reverse'
        return_value = 'http://example.com/dummy'

        with patch(func, return_value=return_value) as mock_reverse:
            url = content_type_url('dummy_url', dummy)

            mock_reverse.assert_called_once_with(
                'dummy_url',
                kwargs={'app_label': 'myapp', 'model_name': 'dummy'}
            )

            self.assertEqual(url, 'http://example.com/dummy')

    def test_query_param_url(self) -> None:
        request = self.factory.get('/some_path', {'foo': 'bar', 'baz': 'qux'})
        context = RequestContext(request, {})

        func = 'django_spire.core.templatetags.spire_core_tags.reverse'
        return_value = 'http://example.com/dummy'

        with patch(func, return_value=return_value) as mock_reverse:
            url = query_param_url(context, 'dummy_url')
            mock_reverse.assert_called_once_with('dummy_url', kwargs={})

            self.assertTrue(url.startswith('http://example.com/dummy?'))
            self.assertIn('foo=bar', url)
            self.assertIn('baz=qux', url)


class TemplateRenderingTests(TestCase):
    def test_render_add_str_filter(self) -> None:
        template_code = """
            {% load spire_core_tags %}

            {{ "Hello" | add_str:" World" }}
        """

        tmpl = Template(template_code)
        rendered = tmpl.render(Context())
        self.assertIn('Hello World', rendered)

    def test_render_to_snake_case_tag(self) -> None:
        template_code = """
            {% load spire_core_tags %}

            {% to_snake_case "Hello World" as snake_case %}
            {{ snake_case }}
        """

        tmpl = Template(template_code)
        rendered = tmpl.render(Context())
        self.assertIn('hello_world', rendered)



from __future__ import annotations

from typing_extensions import TYPE_CHECKING

from django.contrib.auth.decorators import login_required
from django.http import JsonResponse

from django_spire.file.interfaces import MultiFileUploader, SingleFileUploader

if TYPE_CHECKING:
    from django.core.handlers.wsgi import WSGIRequest


@login_required()
def file_multiple_upload_ajax(request: WSGIRequest) -> JsonResponse:
    if request.method == 'POST':
        file_uploader = MultiFileUploader(request.POST.get('related_field', ''))
        files = file_uploader.upload(list(request.FILES.values()))

        return JsonResponse({
            'files': [file.to_dict() for file in files]
        })


@login_required()
def file_single_upload_ajax(request: WSGIRequest) -> JsonResponse:
    if request.method == 'POST':
        file_uploader = SingleFileUploader(request.POST.get('related_field', ''))
        file = file_uploader.upload(list(request.FILES.values())[0])

        return JsonResponse({
            'file': file.to_dict()
        })



from __future__ import annotations

import json

from django.core.handlers.wsgi import WSGIRequest
from django.http import JsonResponse

from django_spire.core.decorators import valid_ajax_request_required
from django_spire.knowledge.entry.version.block.models import EntryVersionBlock


@valid_ajax_request_required
def update_text_view(request: WSGIRequest) -> JsonResponse:
    body_data = json.loads(request.body.decode('utf-8'))
    pk = body_data.get('pk')

    try:
        version_block = EntryVersionBlock.objects.get(pk=pk)
    except EntryVersionBlock.DoesNotExist:
        return JsonResponse({'type': 'error', 'message': 'Block Not Found'})

    value = body_data.get('value')
    block_type = body_data.get('block_type')

    if value is None or block_type is None:
        return JsonResponse({'type': 'error', 'message': 'Missing Required Data'})

    version_block.services.processor.update_block(value=value, block_type=block_type)

    return JsonResponse({'type': 'success'})



from __future__ import annotations

from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.entry.version.models import EntryVersion


@AppAuthController('knowledge').permission_required('can_view')
def detail_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    current_version = get_object_or_404(EntryVersion, pk=pk)
    entry = current_version.entry
    version_blocks = current_version.blocks.active().order_by('order')

    def breadcrumbs_func(breadcrumbs):
        breadcrumbs.add_breadcrumb(
            name='Knowledge',
            href=reverse('django_spire:knowledge:page:home')
        )
        collection = entry.collection
        breadcrumbs.add_breadcrumb(
            name=f'{collection.name}'
        )
        breadcrumbs.add_breadcrumb(
            name=f'{entry.name}',
        )

    return portal_views.detail_view(
        request,
        obj=entry,
        breadcrumbs_func=breadcrumbs_func,
        context_data={
            'entry': entry,
            'current_version': current_version,
            'version_blocks': version_blocks,
        },
        template='django_spire/knowledge/entry/version/page/detail_page.html',
    )



from __future__ import annotations

import random

from datetime import timedelta

from django.utils.timezone import localtime

from django_spire.auth.user.models import AuthUser
from django_spire.knowledge.entry.models import Entry
from django_spire.knowledge.entry.version import models

from django_spire.contrib.seeding import DjangoModelSeeder
from django_spire.knowledge.entry.version.block.models import EntryVersionBlock
from django_spire.knowledge.entry.version.block.seeding.constants import SAFETY_BLOCKS
from django_spire.knowledge.entry.version.choices import EntryVersionStatusChoices


class EntryVersionSeeder(DjangoModelSeeder):
    model_class = models.EntryVersion
    cache_name = 'entry_version_seeder'
    fields = {
        'id': 'exclude',
        'entry_id': ('custom', 'fk_random', {'model_class': Entry}),
        'author_id': ('custom', 'fk_random', {'model_class': AuthUser}),
        'published_datetime': 'exclude',
        'last_edit_datetime': (
            'custom',
            'date_time_between',
            {'start_date': '-30d', 'end_date': 'now'},
        ),
        'is_deleted': ('static', False),
        'is_active': ('static', True),
    }

    @classmethod
    def seed_database(
            cls,
            count: int = 1,
            fields: dict | None = None
    ) -> list[models.EntryVersion]:
        entry_versions = super().seed_database(count=count, fields=fields)

        for entry_version in entry_versions:
            if entry_version.status == EntryVersionStatusChoices.PUBLISHED:
                entry_version.published_datetime = localtime() - timedelta(
                    days=random.randint(0, 30), hours=random.randint(0, 23)
                )
        cls.model_class.objects.bulk_update(entry_versions, ['published_datetime'])

        cls._seed_blocks(entry_versions)

        return entry_versions

    @classmethod
    def _seed_blocks(cls, entry_versions: list[models.EntryVersion]):
        for entry_version in entry_versions:
            safety_blocks = random.choice(SAFETY_BLOCKS)

            version_blocks = []
            for idx, safety_block in enumerate(safety_blocks):
                version_block = EntryVersionBlock(
                    version=entry_version,
                    type=safety_block.type,
                    order=idx,
                )
                version_block.block = safety_block
                version_blocks.append(version_block)

            EntryVersionBlock.objects.bulk_create(version_blocks)



# Generated by Django 5.2.6 on 2025-09-23 14:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_auth_group', '0001_initial'),
        ('django_spire_knowledge', '0003_alter_collection_order_alter_entry_order_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='collection',
            options={'permissions': [('can_access_all_collections', 'Can Access All Collections'), ('can_change_collection_groups', 'Can Change Collection Groups')], 'verbose_name': 'Collection', 'verbose_name_plural': 'Collections'},
        ),
        migrations.CreateModel(
            name='CollectionGroup',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('auth_group', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='collection_groups', related_query_name='collection_group', to='django_spire_auth_group.authgroup')),
                ('collection', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='groups', related_query_name='group', to='django_spire_knowledge.collection')),
            ],
        ),
    ]



from __future__ import annotations

from django_spire.contrib.service import BaseDjangoModelService

from typing import TYPE_CHECKING

from django_spire.knowledge.entry.version.block.maps import ENTRY_BLOCK_MAP
from django_spire.knowledge.entry.version.maps import FILE_TYPE_CONVERTER_MAP

if TYPE_CHECKING:
    from django_spire.file.models import File
    from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
    from django_spire.knowledge.entry.version.block.models import EntryVersionBlock
    from django_spire.knowledge.entry.version.models import EntryVersion


class EntryVersionBlockFactoryService(BaseDjangoModelService['EntryVersionBlock']):
    obj: EntryVersionBlock

    def create_blank_block(
            self,
            entry_version: EntryVersion,
            block_type: BlockTypeChoices,
            order: int,
            **kwargs
    ) -> EntryVersionBlock:
        self.obj = self.obj_class(
            version=entry_version,
            type=block_type,
            order=order,
        )
        self.obj.block = ENTRY_BLOCK_MAP[block_type](
            value='',
            type=block_type,
            **kwargs
        )
        self.obj.save()

        self.obj.ordering_services.processor.move_to_position(
            destination_objects=(
                self.obj_class.objects
                .active()
                .by_version_id(entry_version_id=entry_version.id)
            ),
            position=order,
        )

        return self.obj

    def create_blocks_from_file(
            self,
            file: File,
            entry_version: EntryVersion
    ) -> list[EntryVersionBlock]:
        if file.type not in FILE_TYPE_CONVERTER_MAP:
            return []

        converter = FILE_TYPE_CONVERTER_MAP[file.type](entry_version=entry_version)

        return self.obj_class.objects.bulk_create(
            converter.convert_file_to_blocks(file=file)
        )

    def create_null_block(
            self,
            entry_version: EntryVersion,
            block_type: BlockTypeChoices,
            order: int,
            value: str,
            **kwargs,
    ):
        self.obj = self.obj_class(
            version=entry_version,
            type=block_type,
            order=order
        )
        block = ENTRY_BLOCK_MAP[block_type](
            type=block_type,
            value=value,
            **kwargs
        )
        self.obj.block = block
        return self.obj



from __future__ import annotations

from dataclasses import dataclass

from typing_extensions import TYPE_CHECKING

from django.contrib.auth.models import Permission, Group, User
from django.contrib.contenttypes.models import ContentType

from django_spire.auth.group.utils import (
    codename_list_to_perm_level,
    codename_to_perm_level,
    perm_level_to_string
)
from django_spire.auth.permissions.consts import VALID_PERMISSION_LEVELS
from django_spire.core.utils import get_object_from_module_string

if TYPE_CHECKING:
    from django.db.models import QuerySet, Model


class ModelPermission:
    def __init__(
            self,
            name: str,
            model_class_path: str,
            is_proxy_model: bool,
            verbose_name: str | None = None,
    ):
        self.name = name
        self.model_class_path = model_class_path
        self.is_proxy_model = is_proxy_model
        self.verbose_name = verbose_name

    @property
    def model_class(self) -> type[Model]:
        return get_object_from_module_string(self.model_class_path)



class ModelPermissions:
    def __init__(self, model_permission: ModelPermission):
        self.model = model_permission.model_class
        self.is_proxy_model = model_permission.is_proxy_model

        # All the permissions accessible to this model
        self.permissions: QuerySet[Permission] = self._set_model_perms()

    @property
    def app_label(self) -> str:
        # Used to generate codenames. For example delete_app_label
        return self.model._meta.app_label

    @property
    def model_name(self) -> str:
        return self.model._meta.model_name

    def find_permissions_by_level(
        self,
        perm_level: VALID_PERMISSION_LEVELS | None
    ) -> list[Permission]:
        permission_list = []

        for perm in self.permissions:
            if codename_to_perm_level(perm.codename) <= perm_level:
                permission_list.append(perm)

        return permission_list

    def get_special_role(self, codename: str) -> Permission | None:
        for perm in self.permissions:
            if perm.codename == codename:
                return perm

        return None

    def _set_model_perms(self) -> QuerySet[Permission]:
        content_type = ContentType.objects.get_for_model(
            self.model(),
            for_concrete_model=not self.is_proxy_model
        )

        return Permission.objects.filter(content_type=content_type)

    def special_role_list(self) -> list[Permission]:
        return [
            perm
            for perm in self.permissions
            if perm.codename.startswith('can')
        ]


class GroupPermissions:
    def __init__(self, group: Group, model_permission: ModelPermission):
        """
            Helper to use Django Groups as a cascading permission structure.
        """
        self.group = group
        self.model_permissions = ModelPermissions(model_permission)
        self.group_perms = self.group.permissions.all()

    def add_special_role(self, codename: str) -> None:
        special_role = self.model_permissions.get_special_role(codename)

        if special_role is not None:
            self.group.permissions.add(special_role)
            self.set_group_perms()

    def has_special_role(self, codename: str) -> bool:
        for perm in self.group_perms:
            if perm.codename == codename:
                return True

        return False

    def perm_level(self) -> VALID_PERMISSION_LEVELS:
        codename_list = []

        for perm in self.group_perms:
            if perm.codename.split('_')[-1] == self.model_permissions.model_name:
                codename_list.append(perm.codename)

        return codename_list_to_perm_level(codename_list)

    def perm_level_verbose(self) -> str:
        return perm_level_to_string(self.perm_level())

    @staticmethod
    def remove_special_permissions(perm_list: list | QuerySet) -> list:
        return [
            perm for perm in perm_list if not perm.codename.startswith('can')
        ]

    def update_perms(self, perm_level: VALID_PERMISSION_LEVELS) -> None:
        cascading_perms = self.remove_special_permissions(
            self.model_permissions.find_permissions_by_level(perm_level)
        )

        cascading_perms_queryset = Permission.objects.filter(
            pk__in=[perm.pk for perm in cascading_perms]
        )
        self.group.permissions.add(*cascading_perms_queryset)

        cascading_perms_id_list = [perm.pk for perm in cascading_perms]
        prohibited_permissions = self.remove_special_permissions(
            self.model_permissions.permissions.exclude(
                id__in=cascading_perms_id_list
            )
        )

        self.group.permissions.remove(*prohibited_permissions)

        self.set_group_perms()

    def remove_special_role(self, codename: str) -> None:
        special_role = self.model_permissions.get_special_role(codename)

        if special_role:
            self.group.permissions.remove(special_role)
            self.set_group_perms()

    def set_group_perms(self) -> None:
        self.group_perms = self.group.permissions.all()

    def toggle_special_role(self, codename: str) -> None:
        if self.has_special_role(codename):
            self.remove_special_role(codename)
        else:
            self.add_special_role(codename)


class UserPermissionHelper:
    def __init__(self, user: User, model_permission: ModelPermission):
        """
            Helper to query user's current cascading permissions
        """
        self.user = user
        self.model_permissions = ModelPermissions(model_permission)
        self.user_perms = self.get_user_perms()

    def get_user_perms(self) -> QuerySet[Permission]:
        return Permission.objects.filter(group__user=self.user).distinct()

    def perm_level(self) -> VALID_PERMISSION_LEVELS:
        codename_list = []

        for perm in self.user_perms:
            if perm.codename.split('_')[-1] == self.model_permissions.model_name:
                codename_list.append(perm.codename)

        return codename_list_to_perm_level(codename_list)

    def perm_level_verbose(self) -> str:
        return perm_level_to_string(self.perm_level())



# Generated by Django 5.2 on 2025-05-17 23:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_spire_ai_chat', '0002_remove_chatmessage_content_chatmessage__content_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='chatmessage',
            old_name='_content',
            new_name='_intel_data',
        ),
        migrations.RemoveField(
            model_name='chatmessage',
            name='_content_intel_class',
        ),
        migrations.AddField(
            model_name='chatmessage',
            name='_intel_class_name',
            field=models.TextField(default=''),
            preserve_default=False,
        ),
    ]



from __future__ import annotations

from django.db import models

from django_spire.auth.group.models import AuthGroup
from django_spire.contrib.ordering.mixins import OrderingModelMixin
from django_spire.history.mixins import HistoryModelMixin
from django_spire.knowledge.collection.querysets import CollectionQuerySet
from django_spire.knowledge.collection.services.service import CollectionGroupService, \
    CollectionService


class Collection(HistoryModelMixin, OrderingModelMixin):
    parent = models.ForeignKey(
        'self',
        on_delete=models.CASCADE,
        related_name='children',
        related_query_name='child',
        null=True,
        blank=True
    )

    name = models.CharField(max_length=255)
    description = models.TextField()

    services = CollectionService()
    objects = CollectionQuerySet.as_manager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = 'Collection'
        verbose_name_plural = 'Collections'
        db_table = 'django_spire_knowledge_collection'
        permissions = [
            ('can_access_all_collections', 'Can Access All Collections'),
            ('can_change_collection_groups', 'Can Change Collection Groups'),
        ]


class CollectionGroup(models.Model):
    collection = models.ForeignKey(
        Collection,
        on_delete=models.CASCADE,
        related_name='groups',
        related_query_name='group',
    )

    auth_group = models.ForeignKey(
        AuthGroup,
        on_delete=models.CASCADE,
        related_name='collection_groups',
        related_query_name='collection_group',
    )

    services = CollectionGroupService()



from django.urls import path

from django_spire.comment import views


app_name = 'comment'

urlpatterns = [
    path('<int:comment_pk>/<int:obj_pk>/<str:app_label>/<str:model_name>/form/content/',
         views.comment_modal_form_content,
         name='form_content'),

    path('<int:comment_pk>/<int:obj_pk>/<str:app_label>/<str:model_name>/form/',
         views.comment_form_view,
         name='form'),

    path('<int:comment_pk>/<int:obj_pk>/<str:app_label>/<str:model_name>/delete/form/',
         views.comment_modal_delete_form_view,
         name='delete_form'),
]



from django_spire.contrib.seeding.field.base import BaseFieldSeeder


class StaticFieldSeeder(BaseFieldSeeder):
    keyword = "static"

    def seed(self, manager = None, count = 1) -> list:
        return [
            {field_name: value[1] for field_name, value in self.seeder_fields.items()}
            for _ in range(count)
        ]



from __future__ import annotations

from django.core.handlers.wsgi import WSGIRequest
from django.shortcuts import get_object_or_404
from django.template.response import TemplateResponse
from django.urls import reverse

from django_spire.auth.controller.controller import AppAuthController
from django_spire.contrib.generic_views import portal_views
from django_spire.knowledge.collection.models import Collection


@AppAuthController('knowledge').permission_required('can_delete')
def delete_view(request: WSGIRequest, pk: int) -> TemplateResponse:
    collection = get_object_or_404(Collection, pk=pk)

    return portal_views.delete_form_view(
        request,
        obj=collection,
        delete_func=collection.services.processor.set_deleted,
        return_url=request.GET.get(
            'return_url',
            reverse('django_spire:knowledge:page:home')
        )
    )



from __future__ import annotations

from typing import TYPE_CHECKING

from django.core.management.base import CommandError

if TYPE_CHECKING:
    from django_spire.core.management.commands.spire_startapp_pkg.filesystem import FileSystemInterface
    from django_spire.core.management.commands.spire_startapp_pkg.registry import AppRegistryInterface
    from django_spire.core.management.commands.spire_startapp_pkg.reporter import ReporterInterface
    from django_spire.core.management.commands.spire_startapp_pkg.resolver import PathResolverInterface


class AppValidator:
    """
    Validates Django app paths and configurations.

    This class performs validation checks to ensure app paths are properly
    formatted, don't conflict with existing apps, and use valid root apps.
    """

    def __init__(
        self,
        reporter: ReporterInterface,
        registry: AppRegistryInterface,
        path_resolver: PathResolverInterface,
        filesystem: FileSystemInterface
    ):
        """
        Initializes the validator with required dependencies.

        :param reporter: Reporter for displaying error messages.
        :param registry: Registry for checking installed apps.
        :param path_resolver: Path resolver for determining file locations.
        :param filesystem: File system interface for checking file existence.
        """

        self._filesystem = filesystem
        self._path_resolver = path_resolver
        self._registry = registry
        self._reporter = reporter

    def validate_app_format(self, app_path: str) -> None:
        """
        Validates that an app path uses dot notation.

        :param app_path: App path to validate.
        :raises CommandError: If the app path doesn't contain dots.
        """

        if '.' not in app_path:
            self._reporter.write('\n', self._reporter.style_notice)

            message = 'Invalid app name format. The app path must use dot notation (e.g., "parent.child").'
            raise CommandError(message)

    def validate_app_path(self, components: list[str]) -> None:
        """
        Validates that an app path doesn't already exist.

        :param components: List of app path components.
        :raises CommandError: If an app already exists at the destination path.
        """

        destination = self._path_resolver.get_app_destination(components)

        if self._filesystem.has_content(destination):
            self._reporter.write('\n', self._reporter.style_notice)

            message = (
                f'The app already exists at {destination}. '
                'Please remove the existing app or choose a different name.'
            )

            raise CommandError(message)

    def validate_root_app(self, components: list[str]) -> None:
        """
        Validates that the root app component is registered in Django.

        :param components: List of app path components.
        :raises CommandError: If the root app is not a valid registered app.
        """

        valid_roots = self._registry.get_valid_root_apps()
        root = components[0]

        if root not in valid_roots:
            self._reporter.write('\n', self._reporter.style_notice)

            message = (
                f'Invalid root app "{root}". '
                f'Valid root apps: {", ".join(sorted(valid_roots))}.'
            )

            raise CommandError(message)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.utils.timezone import localtime

from django_spire.contrib.service import BaseDjangoModelService
from django_spire.knowledge.entry.version.choices import EntryVersionStatusChoices

if TYPE_CHECKING:
    from django_spire.knowledge.entry.version.models import EntryVersion


class EntryVersionProcessorService(BaseDjangoModelService['EntryVersion']):
    obj: EntryVersion

    def publish(self):
        self.obj.status = EntryVersionStatusChoices.PUBLISHED
        self.obj.published_datetime = localtime()
        self.obj.save()



from __future__ import annotations

from typing_extensions import Any

from django.http import HttpResponse
from django.urls import reverse


def reverse_generic_relation(content_object: Any, **kwargs) -> HttpResponse | None:
    from operator import attrgetter

    model_name = content_object.__class__.__name__.lower()

    CONTENT_OBJECT_URL_MAP = {
        # 'APP_NAME': ('URL', ('pk', 'pk')),
    }

    url_path = None

    if isinstance(CONTENT_OBJECT_URL_MAP[model_name], tuple):
        url_path = CONTENT_OBJECT_URL_MAP[model_name][0]

        for kwarg in CONTENT_OBJECT_URL_MAP[model_name][1:]:
            if kwarg[0] == 'pk':
                if isinstance(kwarg[1], str):
                    kwargs[kwarg[0]] = getattr(content_object, kwarg[1])
                elif isinstance(kwarg[1], int):
                    kwargs[kwarg[0]] = kwarg[1]
            elif kwarg[1] == 'parent_pk':
                retriever = attrgetter(kwarg[2])
                kwargs[kwarg[0]] = retriever(content_object)
    elif isinstance(CONTENT_OBJECT_URL_MAP[model_name], str):
        url_path = CONTENT_OBJECT_URL_MAP[model_name]

    if url_path is not None:
        return reverse(url_path, kwargs=kwargs)

    return HttpResponse('home:page:home')



from django.urls import include, path


app_name = 'knowledge'

urlpatterns = [
    path('page/', include('django_spire.knowledge.urls.page_urls', namespace='page')),
    path('collection/', include('django_spire.knowledge.collection.urls', namespace='collection')),
    path('entry/', include('django_spire.knowledge.entry.urls', namespace='entry')),
]



from __future__ import annotations

from django.db import models
from django.forms import model_to_dict
from django.template.loader import render_to_string

from django_spire.contrib.ordering.mixins import OrderingModelMixin
from django_spire.history.mixins import HistoryModelMixin
from django_spire.knowledge.entry.version.block.choices import BlockTypeChoices
from django_spire.knowledge.entry.version.block.maps import ENTRY_BLOCK_MAP
from django_spire.knowledge.entry.version.block.services.service import EntryVersionBlockService
from django_spire.knowledge.entry.version.block.blocks.block import BaseBlock
from django_spire.knowledge.entry.version.models import EntryVersion
from django_spire.knowledge.entry.version.block.querysets import EntryVersionBlockQuerySet


class EntryVersionBlock(HistoryModelMixin, OrderingModelMixin):
    version = models.ForeignKey(
        EntryVersion,
        on_delete=models.CASCADE,
        related_name='blocks',
        related_query_name='block'
    )
    type = models.CharField(
        max_length=32,
        choices=BlockTypeChoices,
        default=BlockTypeChoices.TEXT
    )
    _block_data = models.JSONField()
    _text_data = models.TextField()

    objects = EntryVersionBlockQuerySet.as_manager()
    services = EntryVersionBlockService()

    @property
    def block(self) -> BaseBlock:
        return ENTRY_BLOCK_MAP[self.type](**self._block_data)

    @block.setter
    def block(self, value: BaseBlock):
        self.type = value.type
        self._block_data = value.model_dump()
        self._text_data = value.render_to_text()

    def render_to_text(self) -> str:
        return self.block.render_to_text()

    class Meta:
        verbose_name = 'Block'
        verbose_name_plural = 'Blocks'
        db_table = 'django_spire_knowledge_entry_version_block'



from django.urls import include, path


app_name = 'mfa'

urlpatterns  = [
    path('', include('django_spire.auth.mfa.urls.page_urls', namespace='page')),
    path('', include('django_spire.auth.mfa.urls.redirect_urls', namespace='redirect')),
]



from __future__ import annotations

from pathlib import Path

from dandy import BaseIntel
from django.conf import settings

_MARKDOWN_OUTPUT_PATH = Path(settings.BASE_DIR, '.markdown')
_PROMPT_OUTPUT_PATH = Path(settings.BASE_DIR, '.prompt_generator_output')


class DandyPromptPythonFileIntel(BaseIntel):
    source_code: str
    file_name: str

    def to_file(self):
        Path(_PROMPT_OUTPUT_PATH).mkdir(parents=True, exist_ok=True)

        with open(Path(_PROMPT_OUTPUT_PATH, self.file_name), 'w') as f:
            f.write(self.source_code)

        print(f'Done ... saved to "{Path(_PROMPT_OUTPUT_PATH, self.file_name)}"')


class TextToMarkdownIntel(BaseIntel):
    markdown_content: str
    file_name: str

    def to_file(self):
        Path(_MARKDOWN_OUTPUT_PATH).mkdir(parents=True, exist_ok=True)

        with open(Path(_MARKDOWN_OUTPUT_PATH, self.file_name), 'w') as f:
            f.write(self.markdown_content)

        print(f'Done ... saved to "{Path(_MARKDOWN_OUTPUT_PATH, self.file_name)}"')



from django_spire.contrib.seeding.field.enums import FieldSeederTypesEnum

import copy

def normalize_seeder_fields(fields: dict) -> dict:
    normalized = {}

    for k, v in fields.items():
        if v in ("exclude", ("exclude",)):
            continue

        if isinstance(v, tuple):
            # Safely clone v[2] if it's a dict
            extra = ()

            if len(v) > 2:
                extra_value = v[2]

                if isinstance(extra_value, dict):
                    extra = (copy.deepcopy(extra_value),)
                else:
                    extra = (extra_value,)

            normalized[k] = (v[0],) + v[1:2] + extra

        elif callable(v):
            normalized[k] = ("callable", v)

        elif isinstance(v, str) and v.lower() in FieldSeederTypesEnum._value2member_map_:
            normalized[k] = (v.lower(),)

        else:
            normalized[k] = ("static", v)

    return normalized



from django_spire.contrib.service.exceptions import ServiceException
from django_spire.contrib.service.django_model_service import BaseDjangoModelService

__all__ = [
    'BaseDjangoModelService',
    'ServiceException',
]



from __future__ import annotations

from django_spire.ai.chat.message_intel import BaseMessageIntel


class KnowledgeMessageIntel(BaseMessageIntel):
    _template: str = 'django_spire/knowledge/message/knowledge_message_intel.html'
    body: str

    def content_to_str(self) -> str:
        return self.body



from django.urls import path

from django_spire.ai.chat.views import message_response_views


app_name = 'response'

urlpatterns = [
    path('new/', message_response_views.response_message_render_view, name='new'),
]



from __future__ import annotations

from abc import abstractmethod, ABC

from dandy import BaseIntel
from django.template.loader import render_to_string


class BaseMessageIntel(BaseIntel, ABC):
    _template: str

    def __init_subclass__(cls):
        super().__init_subclass__()

        if cls._template is None or cls._template == '':
            message = f'{cls.__module__}.{cls.__qualname__}._template must be set'
            raise ValueError(message)

    @abstractmethod
    def content_to_str(self) -> str:
        raise NotImplementedError

    def render_to_string(self, context_data: dict | None = None):
        return render_to_string(
            template_name=self._template,
            context={**self.model_dump(), **(context_data or {})},
        )

    @property
    def template(self) -> str:
        return self._template


class DefaultMessageIntel(BaseMessageIntel):
    _template: str = 'django_spire/ai/chat/message/default_message.html'
    text: str

    def content_to_str(self) -> str:
        return self.text



# Generated by Django 5.2.1 on 2025-05-15 20:20

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('is_active', models.BooleanField(default=True, editable=False)),
                ('is_deleted', models.BooleanField(default=False, editable=False)),
                ('created_datetime', models.DateTimeField(default=django.utils.timezone.localtime, editable=False)),
                ('type', models.CharField(choices=[('app', 'App'), ('email', 'Email'), ('push', 'Push'), ('sms', 'Sms')], default='email', max_length=32)),
                ('title', models.CharField(max_length=124)),
                ('body', models.TextField(default='')),
                ('url', models.CharField(default='', max_length=255)),
                ('status', models.CharField(choices=[('pending', 'Pending'), ('processing', 'Processing'), ('sent', 'Sent'), ('errored', 'Errored'), ('failed', 'Failed')], default='pending', max_length=32)),
                ('status_message', models.TextField(default='')),
                ('priority', models.CharField(choices=[('low', 'Low'), ('medium', 'Medium'), ('high', 'High')], default='low', max_length=32)),
                ('publish_datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('sent_datetime', models.DateTimeField(blank=True, null=True)),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('content_type', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='django_spire_notifications', to='contenttypes.contenttype')),
                ('user', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='notifications', related_query_name='notification', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Notification',
                'verbose_name_plural': 'Notifications',
                'db_table': 'django_spire_notification',
            },
        ),
    ]



from __future__ import annotations

from django_spire.knowledge.intelligence.bots.entry_search_llm_bot import EntrySearchLlmBot
from django_spire.knowledge.intelligence.intel.collection_intel import CollectionIntel
from django_spire.knowledge.intelligence.intel.entry_intel import EntriesIntel, EntryIntel
from django_spire.knowledge.intelligence.intel.message_intel import KnowledgeMessageIntel
from django_spire.knowledge.intelligence.maps.collection_map import get_collection_map_class
from django_spire.knowledge.intelligence.maps.entry_map import get_entry_map_class


class KnowledgeWorkflow:
    @classmethod
    def process(cls, user_input: str) -> KnowledgeMessageIntel:
        CollectionMap = get_collection_map_class()
        collections = CollectionMap().process(user_input).values

        if collections[0] is None:
            return KnowledgeMessageIntel(
                body=(
                    'There was no knowledge related to your request. Please reword it '
                    'and try again.'
                )
            )

        entries = []
        for collection in collections:
            if collection.entry_count > 0:
                EntryMap = get_entry_map_class(collection=collection)
                entries.extend(EntryMap().process(user_input).values)

        entries = [entry for entry in entries if entry is not None]

        if not entries:
            return KnowledgeMessageIntel(
                body=(
                    'There was no knowledge related to your request. Please reword it '
                    'and try again.'
                )
            )

        entry_search_bot = EntrySearchLlmBot()

        entries_intel = EntriesIntel(
            entry_intel_list=[
                EntryIntel(
                    body=entry_search_bot.process(
                        user_input=user_input,
                        entry=entry
                    ),
                    collection_intel=CollectionIntel(name=entry.collection.name)
                )
                for entry in entries
            ]
        )

        return KnowledgeMessageIntel(body=f'Entries: {entries_intel}')



from __future__ import annotations

from dandy import BaseIntel, Bot, Prompt


class MarkdownTextIntel(BaseIntel):
    text: str


class MarkdownFormatLlmBot(Bot):
    llm_intel_class = MarkdownTextIntel

    def process(self, markdown_content: str) -> str:
        markdown_prompt = Prompt()
        markdown_prompt.text(
            'Can you improve the markdown formatting? Do NOT add or change any of the '
            'content.'
        )
        markdown_prompt.text(markdown_content)

        result = self.llm.prompt_to_intel(prompt=markdown_prompt)
        return result.text



from __future__ import annotations

from django_spire.core.management.commands.spire_startapp_pkg.builder import TemplateBuilder
from django_spire.core.management.commands.spire_startapp_pkg.config import (
    AppConfig,
    AppConfigFactory,
    PathConfig,
)
from django_spire.core.management.commands.spire_startapp_pkg.filesystem import (
    FileSystem,
    FileSystemInterface,
)
from django_spire.core.management.commands.spire_startapp_pkg.generator import (
    AppGenerator,
    TemplateGenerator,
)
from django_spire.core.management.commands.spire_startapp_pkg.maps import generate_replacement_map
from django_spire.core.management.commands.spire_startapp_pkg.permissions import PermissionInheritanceHandler
from django_spire.core.management.commands.spire_startapp_pkg.processor import (
    TemplateEngine,
    TemplateProcessor,
)
from django_spire.core.management.commands.spire_startapp_pkg.registry import (
    AppRegistry,
    AppRegistryInterface,
)
from django_spire.core.management.commands.spire_startapp_pkg.reporter import (
    Reporter,
    ReporterInterface,
)
from django_spire.core.management.commands.spire_startapp_pkg.resolver import (
    PathResolver,
    PathResolverInterface,
)
from django_spire.core.management.commands.spire_startapp_pkg.user_input import UserInputCollector
from django_spire.core.management.commands.spire_startapp_pkg.validator import AppValidator


__all__ = [
    'AppConfig',
    'AppConfigFactory',
    'AppGenerator',
    'AppRegistry',
    'AppRegistryInterface',
    'AppValidator',
    'FileSystem',
    'FileSystemInterface',
    'PathConfig',
    'PathResolver',
    'PathResolverInterface',
    'PermissionInheritanceHandler',
    'Reporter',
    'ReporterInterface',
    'TemplateBuilder',
    'TemplateEngine',
    'TemplateGenerator',
    'TemplateProcessor',
    'UserInputCollector',
    'generate_replacement_map',
]



from __future__ import annotations

from django.contrib import admin

from django_spire.knowledge.entry.version.models import EntryVersion


@admin.register(EntryVersion)
class EntryVersionAdmin(admin.ModelAdmin):
    list_display = [
        'entry__name', 'entry__collection', 'author', 'last_edit_datetime',
        'published_datetime', 'is_deleted'
    ]
    list_select_related = ['entry__collection', 'author']
    list_filter = ['status', 'is_deleted', 'is_active']
    search_fields = ['entry__name', 'author__first_name', 'author__last_name']
    ordering = ['-last_edit_datetime']
    autocomplete_fields = ['entry', 'author']



from __future__ import annotations

from django.urls import reverse

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.knowledge.collection.tests.factories import create_test_collection


class CollectionPageUrlsTests(BaseTestCase):
    def setUp(self):
        super().setUp()
        self.test_collection = create_test_collection()

    def test_delete_view_url_path(self):
        response = self.client.get(
            reverse(
                'django_spire:knowledge:collection:page:delete',
                kwargs={'pk': self.test_collection.pk}
            )
        )

        self.assertEqual(response.status_code, 200)

    def test_list_view_url_path(self):
        response = self.client.get(
            reverse('django_spire:knowledge:page:home')
        )

        self.assertEqual(response.status_code, 200)



from django.contrib.auth.models import User
from django.contrib.contenttypes.fields import GenericRelation
from django.db import models

from django_spire.history.activity.models import Activity, ActivitySubscriber


class ActivityMixin(models.Model):
    activities = GenericRelation(
        Activity,
        related_query_name='activity',
        editable=False
    )

    def add_activity(
            self,
            user: User,
            verb: str,
            information: str,
            recipient: User = None,
            subscribers: list[User] | None = None
    ) -> Activity:

        activity = self.activities.create(
            user=user,
            verb=verb,
            information=information,
            recipient=recipient
        )

        if subscribers:
            subscriber_list = [
                ActivitySubscriber(user=subscriber, activity=activity)
                for subscriber in subscribers
            ]

            ActivitySubscriber.objects.bulk_create(subscriber_list)

        return activity

    @property
    def creator(self) -> User:
        return self.activities.earliest('date_time_entered').user

    class Meta:
        abstract = True



import threading

lock = threading.Lock()

from django_spire.profiling.middleware.profiling import ProfilingMiddleware
from django_spire.profiling.panel import ProfilingPanel


__all__ = [
    'ProfilingMiddleware',
    'ProfilingPanel',
    'lock'
]



from __future__ import annotations

from typing import TYPE_CHECKING

from django_spire.contrib.ordering.querysets import OrderingQuerySetMixin
from django_spire.history.querysets import HistoryQuerySet

if TYPE_CHECKING:
    from django.db.models import QuerySet
    from django_spire.knowledge.entry.version.block.models import EntryVersionBlock


class EntryVersionBlockQuerySet(HistoryQuerySet, OrderingQuerySetMixin):
    def by_version_id(self, entry_version_id: int) -> QuerySet[EntryVersionBlock]:
        return self.filter(version_id=entry_version_id)



from __future__ import annotations

import json
import time

from typing import TYPE_CHECKING

from django.http import StreamingHttpResponse

from django_spire.contrib.progress.tracker import ProgressTracker

if TYPE_CHECKING:
    from typing import Callable


def sse_stream_view(
    key: str,
    interval: float = 0.5,
    should_continue: Callable[[dict], bool] | None = None
) -> StreamingHttpResponse:
    def event_stream() -> None:
        tracker = ProgressTracker(key)
        previous_progress = -1

        while True:
            time.sleep(interval)
            data = tracker.get()

            if data:
                current_progress = data.get('progress', 0)

                if current_progress != previous_progress:
                    yield f'data: {json.dumps(data)}\n\n'
                    previous_progress = current_progress

                if should_continue and not should_continue(data):
                        break

                if current_progress >= 100 or data.get('step') == 'error':
                    break

    response = StreamingHttpResponse(
        event_stream(),
        content_type='text/event-stream'
    )

    response['Cache-Control'] = 'no-cache'
    response['X-Accel-Buffering'] = 'no'

    return response



from __future__ import annotations

from django_spire.knowledge.collection.models import Collection


def create_test_collection(**kwargs) -> Collection:
    data = {
        'parent': None,
        'name': 'Video Game Cheat Codes',
        'description': 'A collection of video game cheat codes.',
        'is_deleted': False,
        'is_active': True
    }
    data.update(kwargs)
    return Collection.objects.create(**data)



from __future__ import annotations

import shutil

from typing import Protocol, TYPE_CHECKING

if TYPE_CHECKING:
    from pathlib import Path
    from typing import Iterator


class FileSystemInterface(Protocol):
    """
    Protocol defining the interface for file system operations.

    This protocol specifies the methods required for interacting with
    the file system during app creation.
    """

    def copy_tree(self, src: Path, dst: Path) -> None: ...
    def create_directory(self, path: Path) -> None: ...
    def exists(self, path: Path) -> bool: ...
    def has_content(self, path: Path) -> bool: ...
    def iterate_files(self, path: Path, pattern: str) -> Iterator[Path]: ...
    def read_file(self, path: Path) -> str: ...
    def rename(self, old: Path, new: Path) -> None: ...
    def write_file(self, path: Path, content: str) -> None: ...


class FileSystem:
    """
    Implementation of file system operations for app generation.

    This class provides concrete implementations of file system operations
    needed to create directories, copy templates, and manage files.
    """

    def copy_tree(self, src: Path, dst: Path) -> None:
        """
        Copies an entire directory tree from source to destination.

        Ignores Python cache directories and compiled files during the copy.

        :param src: Source directory path to copy from.
        :param dst: Destination directory path to copy to.
        """

        shutil.copytree(
            src,
            dst,
            dirs_exist_ok=True,
            ignore=shutil.ignore_patterns('__pycache__', '*.pyc')
        )

    def create_directory(self, path: Path) -> None:
        """
        Creates a directory and all necessary parent directories.

        :param path: Directory path to create.
        """

        path.mkdir(parents=True, exist_ok=True)

    def exists(self, path: Path) -> bool:
        """
        Checks if a path exists on the file system.

        :param path: Path to check for existence.
        :return: True if the path exists, False otherwise.
        """

        return path.exists()

    def has_content(self, path: Path) -> bool:
        """
        Checks if a directory exists and contains any files or subdirectories.

        :param path: Directory path to check.
        :return: True if the directory exists and has content, False otherwise.
        """

        return self.exists(path) and any(path.iterdir())

    def iterate_files(self, path: Path, pattern: str) -> Iterator[Path]:
        """
        Recursively finds all files matching a pattern in a directory.

        :param path: Directory path to search within.
        :param pattern: Glob pattern to match files (e.g., '*.py', '*.template').
        :return: Iterator of matching file paths.
        """

        return path.rglob(pattern)

    def read_file(self, path: Path) -> str:
        """
        Reads the entire contents of a text file.

        :param path: File path to read.
        :return: File contents as a string.
        """

        with open(path, 'r', encoding='utf-8') as handle:
            return handle.read()

    def rename(self, old: Path, new: Path) -> None:
        """
        Renames a file or directory.

        :param old: Current path of the file or directory.
        :param new: New path for the file or directory.
        """

        old.rename(new)

    def write_file(self, path: Path, content: str) -> None:
        """
        Writes content to a text file.

        :param path: File path to write to.
        :param content: String content to write to the file.
        """

        with open(path, 'w', encoding='utf-8') as handle:
            handle.write(content)



from __future__ import annotations

from django_spire.core.tests.test_cases import BaseTestCase
from django_spire.auth.user.tests.factories import create_user


class UserProfileTestCase(BaseTestCase):
    def setUp(self):
        self.user = create_user(username='Wesley', password='goat_99')



from __future__ import annotations

from dandy import Bot, Prompt
from dandy.intel.intel import DefaultIntel

from django_spire.knowledge.entry.models import Entry


class EntrySearchLlmBot(Bot):
    @classmethod
    def process(cls, user_input: str, entry: Entry) -> str:
        entry_prompt = Prompt()
        entry_prompt.text('You are a helpful assistant that helps users find information about entries.')
        entry_prompt.text(f'User Input: {user_input}')
        entry_prompt.text(f'Entry: {entry.name}')

        for version_block in entry.current_version.blocks.all():
            entry_prompt.text(version_block.render_to_text())

        return cls().llm.prompt_to_intel(
            prompt=entry_prompt,
            intel_class=DefaultIntel
        ).content
